//===============================================================================
// Q4Tech Engineering Team
// Mobile Updater Application Block - November 2006
//===============================================================================
// Copyright  Q4Tech Informatica Argentina.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;

namespace Q4Tech.Engineering.Ipc
{
    /// <summary>
    /// First 16 bytes: ClientId.
    /// Next 16 bytes: MessageId.
    /// Next 4 bytes: Data Size.
    /// </summary>
    /// <typeparam name="TData"></typeparam>
    public class IpcCallMessage<TData>
    {
        private Guid clientId;
        private Guid messageId;
        private TData data;
        private int chunkSize;
        private MemoryStream dataStream;
        private int dataSizeLeft;

        public IpcCallMessage()
        {
            dataStream = new MemoryStream();
        }

        public IpcCallMessage(Guid clientId, Guid messageId, TData data, int chunkSize)
        {
            this.clientId = clientId;
            this.messageId = messageId;
            this.Data = data;

            if (chunkSize < 37)
                throw new ArgumentException("Chunk size must be greater than 36");

            this.chunkSize = chunkSize;
        }

        public int ChunkSize
        {
            get { return chunkSize; }
        }

        public Guid ClientId
        {
            get { return clientId; }
        }

        public Guid MessageId
        {
            get { return messageId; }
        }

        public TData Data
        {
            get { return data; }
            set
            {
                this.data = value;

                dataStream = new MemoryStream();

                XmlSerializer serializer = new XmlSerializer(typeof(TData));
                serializer.Serialize(dataStream, data);
                dataStream.Seek(0, SeekOrigin.Begin);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="?"></param>
        /// <returns>
        /// false if the data is completed
        /// true if there is more data to add
        /// An exception is thrown if the messageId or the clientId doesnt match</returns>
        public bool Add(byte[] chunk)
        {
            Guid chunkClientId;
            Guid chunkMessageId;

            MemoryStream chunkStream = new MemoryStream(chunk);
            BinaryReader chunkReader = new BinaryReader(chunkStream);

            chunkClientId = new Guid(chunkReader.ReadBytes(16));

            if (clientId == Guid.Empty)
                clientId = chunkClientId;
            else if (clientId != chunkClientId)
                    throw new InvalidOperationException("Invalid client id for the specified chunk");

            chunkMessageId = new Guid(chunkReader.ReadBytes(16));

            if (messageId == Guid.Empty)
                messageId = chunkMessageId;
            else if (messageId != chunkMessageId)
                throw new InvalidOperationException("Invalid message id for the specified chunk");

            if (dataStream.Position == 0)
            {
                dataSizeLeft = chunkReader.ReadInt32();
            }

            dataStream.Write(chunk, (int)chunkStream.Position, (int)chunk.Length - (int)chunkStream.Position);
            dataSizeLeft -= (int)chunk.Length - (int)chunkStream.Position;

            if (dataSizeLeft < 1)
            {
                XmlSerializer serializer = new XmlSerializer(typeof(TData));
                dataStream.Seek(0, SeekOrigin.Begin);
                data = (TData)serializer.Deserialize(dataStream);
                return false;
            }

            return true;
        }

        //public byte[] Serialize()
        //{
        //    MemoryStream memoryStream = new MemoryStream();
        //    XmlSerializer serializer = new XmlSerializer(this.GetType());
        //    serializer.Serialize(memoryStream, this);
        //    return memoryStream.ToArray();
        //}

        //public static IpcCallMessage<TData> Deserialize(byte[] buffer)
        //{
        //    MemoryStream memoryStream = new MemoryStream(buffer);
        //    XmlSerializer serializer = new XmlSerializer(typeof(IpcCallMessage<TData>));
        //    return (IpcCallMessage<TData>)serializer.Deserialize(memoryStream);
        //}

        public byte[] GetNextChunk()
        {
            bool first;

            if (dataStream.Position == (int)dataStream.Length)
            {
                return null;
            }

            first = dataStream.Position == 0;

            MemoryStream chunkStream = new MemoryStream(chunkSize);
            BinaryWriter chunkWriter = new BinaryWriter(chunkStream);

            chunkWriter.Write(clientId.ToByteArray());
            chunkWriter.Write(messageId.ToByteArray());

            if (first)
                chunkWriter.Write((int)dataStream.Length);

            int bytesToCopy = (int)dataStream.Position + chunkSize - (int)chunkStream.Position <= dataStream.Length ? chunkSize - (int)chunkStream.Position : (int)dataStream.Length - (int)dataStream.Position;

            chunkStream.Write(dataStream.GetBuffer(), (int)dataStream.Position, bytesToCopy);

            dataStream.Position += bytesToCopy;

            return chunkStream.ToArray();
        }
    }
}
