//============================================================================================================
// Microsoft Updater Application Block for .NET
//  http://msdn.microsoft.com/library/en-us/dnbda/html/updater.asp
//	
// Manifest.cs
//
// Contains the implementation of the manifest class.
// 
// For more information see the Updater Application Block Implementation Overview. 
// 
//============================================================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//============================================================================================================

using System;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Text;
using System.Globalization;
using System.Security.Permissions;
using Microsoft.ApplicationBlocks.Mobile.Updater.Configuration;
using System.Collections;
using System.Collections.Generic;

namespace Microsoft.ApplicationBlocks.Mobile.Updater
{
	/// <summary>
	/// This class mantains all the information of the manifest that was retrieved using the URI.
	/// </summary>
    //[Serializable]
	public class Manifest
	{
		#region Private fields

		/// <summary>
		/// The manifest class that was deserialized.
		/// </summary>
		private Xsd.Manifest updateManifest;

		/// <summary>
		/// The manifest id.
		/// </summary>
		private Guid updaterManifestId = Guid.Empty;

		/// <summary>
		/// The collection of files that was downloaded.
		/// </summary>
		private FilesCollection filesToUpdate;

		/// <summary>
		/// The configuration for the downloader included in the manifest.
		/// </summary>
		private Xsd.ManifestDownloaderProviderData downloaderConfigInformation;

		/// <summary>
		/// The application information for the manifest.
		/// </summary>
		private ApplicationManifest manifestOfApplicationToUpdate;

		/// <summary>
		/// The information about the activation processors configured in the manifest.
		/// </summary>
		private List<Xsd.ActivationProcessorProviderData> activationProcessorsConfigInformation;

		private List<Xsd.ConditionEvaluatorProviderData> preDonwloadConditionsConfigInformation;

        private List<Xsd.ConditionEvaluatorProviderData> installConditionsConfigInformation;

		/// <summary>
		/// Indicates whether the manifest will be applied.
		/// </summary>
		private bool apply;

		#endregion

		#region Public Constructors

		/// <summary>
		/// Creates a manifest using the deserialized manifest instance.
		/// </summary>
		/// <param name="manifest">The XSD manifest to wrap.</param>
		public Manifest( Xsd.Manifest manifest )
		{
			updateManifest= manifest;
		}

        /// <summary>
        /// Default constructor
        /// </summary>
        public Manifest()
        {
        }

        /// <summary>
        /// XML Serialization
        /// </summary>
        public Xsd.Manifest UpdateManifest
        {
            get { return updateManifest; }
            set { updateManifest = value; }
        }
	
		#endregion

		#region Manifest Members

		/// <summary>
		/// The manifest ID.
		/// </summary>
        [XmlIgnore]
		public Guid ManifestId
		{
			get
			{
				if ( updaterManifestId == Guid.Empty )
					updaterManifestId = new Guid( updateManifest.ManifestId );
				return updaterManifestId;
			}
		} 

		/// <summary>
		/// Initalizes the manifest by setting the XmlNode for the configuration.
		/// </summary>
		/// <param name="config">The node containinf the manifest representation</param>
        //[SecurityPermission(SecurityAction.Demand, SerializationFormatter=true)]
		public void Init(XmlNode config)
		{
			XmlSerializer xser = new XmlSerializer( typeof( Xsd.Manifest ) );
			updateManifest = (Xsd.Manifest)xser.Deserialize( new XmlNodeReader( config ) );
		}

		/// <summary>
		/// The description of the manifest.
		/// </summary>
        [XmlIgnore]
		public string Description
		{
			get { return updateManifest.Description; }
		}

		/// <summary>
		/// Indicates whether the manifest will be applied.
		/// </summary>
		public bool Apply
		{
			get { return apply; }
			set { apply = value; }
		} 

		/// <summary>
		/// Indicates whether the manifest is mandatory.
		/// </summary>
        [XmlIgnore]
		public bool Mandatory
		{
			get
			{
				return updateManifest.Mandatory == Xsd.YesNoBoolType.True
                    || updateManifest.Mandatory == Xsd.YesNoBoolType.Yes;
			}
		}

		/// <summary>
		/// The collection of files defined in the manifest.
		/// </summary>
        [XmlIgnore]
		public FilesCollection Files
		{
			get
			{
				if ( filesToUpdate == null )
					filesToUpdate = new FilesCollection( updateManifest.Files );
				return filesToUpdate;
			}
		}

        public void RemoveFile( int index )
        {
            Files.RemoveAt(index);
            IEnumerator files = Files.GetEnumerator();  
            List<Xsd.ManifestFile> xsdFiles = new List<Xsd.ManifestFile>();
            while (files.MoveNext())
            {
                FileManifest file = (FileManifest)files.Current;
                Xsd.ManifestFile xsdfile = new Xsd.ManifestFile();
                xsdfile.AnyAttr = file.Attributes;
                xsdfile.Hash = file.Hash;
                xsdfile.Source = file.Source;
                xsdfile.Transient = file.Transient? Xsd.YesNoBoolType.True : Xsd.YesNoBoolType.False;
                xsdFiles.Add( xsdfile );
            }
            updateManifest.Files.Files = xsdFiles.ToArray();                        
        } 

		/// <summary>
		/// The configuration for the downloader defined in the manifest.
		/// </summary>
        [XmlIgnore]
		public Xsd.ManifestDownloaderProviderData Downloader
		{
            get 
            {
                if (updateManifest.Downloader != null)
                    this.downloaderConfigInformation = updateManifest.Downloader;
                else if (this.downloaderConfigInformation == null)
                    this.downloaderConfigInformation = new Microsoft.ApplicationBlocks.Mobile.Updater.Xsd.ManifestDownloaderProviderData();

                return this.downloaderConfigInformation; 
            }

            //get { return updateManifest.Downloader; }
		} 

		/// <summary>
		/// The information for the application.
		/// </summary>
        [XmlIgnore]
		public ApplicationManifest Application
		{
			get
			{
				if ( manifestOfApplicationToUpdate == null )
					manifestOfApplicationToUpdate = new ApplicationManifest( updateManifest.Application );
				return manifestOfApplicationToUpdate;
			}
		} 

		/// <summary>
		/// The activation processors configuration.
		/// </summary>
        [XmlIgnore]
		public List<Xsd.ActivationProcessorProviderData> ActivationProcessors
		{
			get
			{
				if ( activationProcessorsConfigInformation == null )
				{
					activationProcessorsConfigInformation =
                        new List<Xsd.ActivationProcessorProviderData>();
					if (updateManifest.ActivationProcess != null)
						if (updateManifest.ActivationProcess.Tasks != null)
							activationProcessorsConfigInformation.AddRange(
                                updateManifest.ActivationProcess.Tasks);
				}
				return activationProcessorsConfigInformation;
			}
		} 

        /// <summary>
		/// The pre download conditions configuration.
		/// </summary>
        [XmlIgnore]
		public List<Xsd.ConditionEvaluatorProviderData> PreDownloadConditions
		{
			get
			{
				if ( preDonwloadConditionsConfigInformation == null )
				{
					preDonwloadConditionsConfigInformation =
                        new List<Xsd.ConditionEvaluatorProviderData>();
					if (updateManifest.PreDownloadConditions != null)
						if (updateManifest.PreDownloadConditions.Conditions != null)
							preDonwloadConditionsConfigInformation.AddRange(
                                updateManifest.PreDownloadConditions.Conditions);
				}
				return preDonwloadConditionsConfigInformation;
			}
		} 

        /// <summary>
		/// The install conditions configuration.
		/// </summary>
        [XmlIgnore]
		public List<Xsd.ConditionEvaluatorProviderData> InstallConditions
		{
			get
			{
				if ( installConditionsConfigInformation == null )
				{
					installConditionsConfigInformation =
                        new List<Xsd.ConditionEvaluatorProviderData>();
					if (updateManifest.InstallConditions != null)
						if (updateManifest.InstallConditions.Conditions != null)
							installConditionsConfigInformation.AddRange(
                                updateManifest.InstallConditions.Conditions);
				}
				return installConditionsConfigInformation;
			}
		}

		#endregion
	}
}
