//============================================================================================================
// Microsoft Updater Application Block for .NET
//  http://msdn.microsoft.com/library/en-us/dnbda/html/updater.asp
//	
// ManifestManager.cs
//
// Contains the implementation of the manifest manager.
// 
// For more information see the Updater Application Block Implementation Overview. 
// 
//============================================================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//============================================================================================================

using System;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Net;
using System.Reflection;
//using System.Security.Permissions;
//using System.Security.Principal;
using System.Xml;
using System.Xml.Serialization;
using Microsoft.ApplicationBlocks.Mobile.Updater.Configuration;
using Microsoft.ApplicationBlocks.Mobile.Updater.Utilities;
using Microsoft.ApplicationBlocks.Mobile.Updater.Xsd;
using System.Web;

namespace Microsoft.ApplicationBlocks.Mobile.Updater
{
    /// <summary>
    /// This class is responsible for fetching the manifest from the specified location, 
    /// checking if the manifest has been applied and returning a list of valid manifests to
    /// apply.
    /// This class can be used as an instance member or as a static member. It is used as a static 
    /// member variable in the UpdaterManager. In order to support testability of the class we allow
    /// classes to use this as an instance member.
    /// </summary>
    internal sealed class ManifestManager
    {
        public const string ManifestDirectory = "applications";
        public const string ManifestExtension = "appliedmanifest";
        public const string ManifestSchema = "Microsoft.ApplicationBlocks.Mobile.Updater.Xsd.Manifest.xsd";

        #region Constructor

        /// <summary>
        /// Default constructor.
        /// </summary>
        public ManifestManager()
        {
        }
        #endregion

        #region Public Methods

        /// <summary>
        /// Return all the manifests for the Uri and the application id.
        /// </summary>
        /// <param name="location">The Uri location for the manifest.</param>
        /// <param name="applicationId">The application id.</param>
        /// <returns>The manifest list.</returns>
        public Manifest[] GetManifests(Uri location, string applicationId)
        {
            if (String.IsNullOrEmpty(applicationId))
                Logger.LogAndThrowException(new ArgumentException(Resource.ResourceManager[Resource.MessageKey.ManifestManagerMustBeValidId], "applicationId"));

            return this.GetValidManifests(location, applicationId);
        }

        /// <summary>
        /// Save the manifest in the manifest registry so they can be retrieved in the next update.
        /// </summary>
        /// <param name="manifest">The Manifest instance.</param>
        //[SecurityPermission(SecurityAction.Demand)]
        public void CommitUpdate(Manifest manifest)
        {
            // CHANGED
            string manifestPath = Path.Combine(RegistryManager.Instance.BasePath, ManifestDirectory);

            manifestPath = Path.Combine(manifestPath, manifest.Application.ApplicationId);
            if (!Directory.Exists(manifestPath))
            {
                Directory.CreateDirectory(manifestPath);
            }
            manifestPath = Path.Combine(manifestPath, manifest.ManifestId.ToString() + "." + ManifestExtension);

            try
            {
                using (Stream stream = new FileStream(manifestPath, FileMode.OpenOrCreate, FileAccess.Write, FileShare.Read))
                {
                    XmlSerializer xmlSerializer = new XmlSerializer(typeof(Manifest));
                    xmlSerializer.Serialize(stream, manifest);
                }
            }
            catch (Exception ex)
            {
                File.Delete(manifestPath);
                Logger.LogAndThrowException(ex);
            }
        }

        #endregion

        #region Private Members

        /// <summary>
        /// Return the Manifest instance for the application id using the remote location.
        /// </summary>
        /// <param name="location">The Uri to the location of the manifest.</param>
        /// <param name="applicationId">The application id.</param>
        /// <returns></returns>
        private Manifest[] GetValidManifests(Uri location, string applicationId)
        {
            if (location == null)
            {
                Logger.LogAndThrowException(new ArgumentNullException(Resource.ResourceManager[Resource.MessageKey.ArgumentMissing1, "location"]));
            }

            // Donwload and validate the primary manifest
            Xsd.Manifest source = (Xsd.Manifest)ValidateAndDeserialize(typeof(Xsd.Manifest), location, ManifestSchema);

            // Download, validate and process each manifest included by the primary manifest
            int count = 0;
            if (source.IncludedManifests != null)
                count = source.IncludedManifests.Manifest.Length;

            // Allocate an array for holding the primary + included manifests
            ArrayList validManifests = new ArrayList(count + 1);

            // Create the manifest from the Xsd class
            Manifest root = new Manifest(source);

            if (new Guid(root.Application.ApplicationId) == new Guid(applicationId))
            {
                PopulateValidManifestList(root, source, validManifests);

                // Check for duplicate included manifests.
                ArrayList includedManifestIds = new ArrayList(count);
                for (int i = 0; i < count; i++)
                {
                    string id = source.IncludedManifests.Manifest[i].ManifestId;
                    if (includedManifestIds.Contains(id))
                        Logger.LogAndThrowException(new ApplicationUpdaterException(Resource.ResourceManager[Resource.MessageKey.DuplicatedManifestId]));
                    includedManifestIds.Add(id);
                }
                // Parse included manifests
                for (int i = 0; i < count; i++)
                {
                    try
                    {
                        string absoulteUri = location.AbsoluteUri;
                        string uriPathPart = absoulteUri.Substring(0, absoulteUri.LastIndexOf('/'));
                        Uri uri = new Uri(uriPathPart + source.IncludedManifests.Manifest[i].Location);

                        Xsd.Manifest include = (Xsd.Manifest)ValidateAndDeserialize(typeof(Xsd.Manifest), uri, ManifestSchema);
                        Manifest child = new Manifest(include);
                        if (String.Compare(child.Application.ApplicationId, applicationId, true, CultureInfo.InvariantCulture) == 0)
                        {
                            PopulateValidManifestList(child, source, validManifests);
                        }
                    }
                    catch (Exception ex)
                    {
                        Logger.LogAndThrowException(ex);
                    }
                }
            }
            // FIXME: else validManifests is not really valid!
            return (Manifest[])validManifests.ToArray(typeof(Manifest));
        }

        /// <summary>
        /// Check wether the manifest is applied locally and add the manifest to a list or not.
        /// </summary>
        /// <param name="manifestToApply">The instance of the Manifest to apply.</param>
        /// <param name="primaryManifest">The primary manifest instance.</param>
        /// <param name="validManifests">The valid manifest list.</param>
        private void PopulateValidManifestList(Manifest manifestToApply, Xsd.Manifest primaryManifest, ArrayList validManifests)
        {
            if (ManifestApplied(manifestToApply.Application.ApplicationId, manifestToApply.ManifestId))
                return;
            manifestToApply.Apply =
                   primaryManifest.Mandatory == YesNoBoolType.True
                || primaryManifest.Mandatory == YesNoBoolType.Yes;

            // Make a hash comparison
            if (manifestToApply.Files.IsHashComparisonEnabled)
            {
                RemoveAppliedFiles(manifestToApply);
            }
            if (manifestToApply.Files.Count > 0)
            {
                validManifests.Add(manifestToApply);
            }
        }

        /// <summary>
        /// Removes the manifest from the list of applied manifests.
        /// </summary>
        /// <param name="manifest">The instance to the Manifest.</param>
        private void RemoveAppliedFiles(Manifest manifest)
        {
            string rootPath = manifest.Application.Location;

            for (int i = 0; i < manifest.Files.Count; i++)
            {
                FileManifest file = manifest.Files[i];

                string filePath = Path.Combine(rootPath, file.Source);

                if (file.Hash == null)
                {
                    Logger.LogAndThrowException(new ApplicationUpdaterException(Resource.ResourceManager[Resource.MessageKey.FileHashIsNull, file.Source]));
                }

                if (FileNotChanged(manifest.Files.HashProvider, file.Hash, filePath))
                {
                    manifest.RemoveFile(i);
                    i--;
                }
            }
        }

        /// <summary>
        /// Check the manifest file hash with the calculated file hash to detect file changes.
        /// </summary>
        /// <param name="hashProviderName">The EntLib hash provider name.</param>
        /// <param name="hash">The calculated hash in the manifest.</param>
        /// <param name="file">The flie that will be checked.</param>
        /// <returns><c>true</c>, if the file has not changed (i.e., matches the declared hash).</returns>
        private bool FileNotChanged(string hashProviderName, string hash, string file)
        {
            return File.Exists(file) && FileHashHelper.CompareHash(hashProviderName, file, hash);
        }

        /// <summary>
        /// Download and validates the manifest using the Uri location.
        /// </summary>
        /// <param name="type">The type that represents the manifest class.</param>
        /// <param name="location">The location of the manifest.</param>
        /// <param name="schemaResource">The resource where the embedded XSD resides in the assembly.</param>
        /// <returns>The Manifest instance.</returns>
        //[SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
        private object ValidateAndDeserialize(Type type, Uri location, string schemaResource)
        {
            object result = null;
            try
            {
                string doc = DownloadFile(location);
                using (Stream schema = Assembly.GetExecutingAssembly().GetManifestResourceStream(schemaResource))
                {
                    SchemaValidator validator = new SchemaValidator(doc, schema);
                    if (!validator.Validate())
                    {
                        Logger.LogAndThrowException(new ApplicationUpdaterException(Resource.ResourceManager[Resource.MessageKey.ManifestSchemaError, location, schemaResource]));
                    }

                    XmlSerializer xser = new XmlSerializer(type);
                    result = xser.Deserialize(new XmlTextReader(doc, XmlNodeType.Document, null));
                }
            }
            catch (Exception ex)
            {
                Logger.LogAndThrowException(ex);
            }
            return result;
        }

        /// <summary>
        /// Used to detect whether the manifest is applied or not.
        /// </summary>
        /// <param name="applicationID">The application id.</param>
        /// <param name="manifestID">The manifest id.</param>
        /// <returns><c>true</c>, if the manifest is applied, otherwise returns <c>false</c>.</returns>
        private bool ManifestApplied(string applicationID, Guid manifestID)
        {
            // Check if the manifest has already been applied
            // CHANGED
            string manifestPath = Path.Combine(RegistryManager.Instance.BasePath, ManifestDirectory);
            manifestPath = Path.Combine(manifestPath, applicationID);
            manifestPath = Path.Combine(manifestPath, manifestID.ToString() + "." + ManifestExtension);

            // If the manifest file exists, the manifest has been applied
            // Or it's being applied if there's an associated task related to the manifest
            return File.Exists(manifestPath) || RegistryManager.Instance.GetByManifestID(manifestID) != null;
        }

        /// <summary>
        /// Download a file using HTTP by a given Uri.
        /// </summary>
        /// <param name="uri">The uri of the file to download.</param>
        /// <returns>The contents of the file.</returns>
        private string DownloadFile(Uri uri)
        {
            WebRequest request = WebRequest.Create(uri);

            AuthenticationType method = AuthenticationType.Anonymous;
            switch (method)
            {
                //case AuthenticationType.Integrated:
                //    {
                //        throw new NotImplementedException();
                //        break;
                //    }
                //case AuthenticationType.Basic:
                //    {
                //        string windowsAuthType = "Basic";

                //        CredentialCache cache = new CredentialCache();
                //        cache.Add(
                //            uri,
                //            windowsAuthType,
                //            new NetworkCredential(
                //            new UpdaterConfigurationView().ManifestManager.UserName,
                //            new UpdaterConfigurationView().ManifestManager.Password));

                //        request.Credentials = cache;
                //        request.PreAuthenticate = true;
                //        break;
                //    }
                case AuthenticationType.Anonymous:
                    {
                        request.Credentials = CredentialCache.DefaultCredentials;
                        break;
                    }

            }

            string response = String.Empty;

            using (WebResponse res = request.GetResponse())
            {
                using (StreamReader reader = new StreamReader(res.GetResponseStream(), true))
                {
                    response = reader.ReadToEnd();
                }
            }
            return response;
        }

        #endregion
    }
}