//============================================================================================================
// Microsoft Updater Application Block for .NET
//  http://msdn.microsoft.com/library/en-us/dnbda/html/updater.asp
//	
// UpdaterTask.cs
//
// Contains the implementation of the updater task.
// 
// For more information see the Updater Application Block Implementation Overview. 
// 
//============================================================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//============================================================================================================

using System;
using System.IO;
using System.Security.Permissions;
using Microsoft.ApplicationBlocks.Mobile.Updater.Configuration;

namespace Microsoft.ApplicationBlocks.Mobile.Updater
{
	/// <summary>
	/// Holds the state information about an update in progress.
	/// </summary>
	public class UpdaterTask
	{
		#region Private fields

		/// <summary>
		/// The task Id.
		/// </summary>
        private Guid id = Guid.NewGuid();

		/// <summary>
		/// The base folder where files will be downloaded.
		/// </summary>
		private string downloadFilesBase;

        /// <summary>
        /// The full path to the base folder where files will be downloaded.
        /// </summary>
        private string downloadFilesPath;

		/// <summary>
		/// The manifest that is being processed.
		/// </summary>
		private Manifest manifest;

		/// <summary>
		/// The task context where all the components can set information for later retrieval.
		/// </summary>
		private StateBag context = new StateBag();

		/// <summary>
		/// The status of the updater task.
		/// </summary>
		private UpdaterTaskState state = UpdaterTaskState.None;

		/// <summary>
		/// An object that can be used to synchronize access to the UpdaterTask.
		/// </summary>
		private object syncRoot = new object();

		#endregion

		#region Constructors

        /// <summary>
        /// Default (Parameterless) constructor,
        /// added for serialization porpouses only
        /// </summary>
        public UpdaterTask()
        {
        }

		/// <summary>
		/// Creates an instance of the updater task using the specified manifest.
		/// </summary>
		/// <param name="manifest">The manifest corresponding to an updater task.</param>
		public UpdaterTask( Manifest manifest )
		{
            this.Manifest = manifest;
		}

		#endregion

		#region UpdaterTask Members

		/// <summary>
		/// A property bag where components can set key-value pairs of information for later retrieval.
        /// Reduced only to string-bool pairs
		/// </summary>
		/// <param name="key">The index key of the item to locate.</param>
		public object this[ string key ]
		{
			get
			{
                return context.ContainsKey(key) ? context[key] : null;
			}
			set
			{
                context[key] = value;
			}
		}
	
        /// <summary>
        /// XML Serialization
        /// </summary>
        public StateBag Context
        {
            get { return context; }
            set { context = value; }
        }

		/// <summary>
		/// Gets an object that can be used to synchronize access to the UpdaterTask.
		/// </summary>
		public object SyncRoot
		{
			get { return syncRoot; }
		}

        /// <summary>
        /// The manifest corresponding to the current UpdaterTask.
        /// </summary>
        public Manifest Manifest
        {
            get
            {
                return manifest;
            }
            set
            {
                manifest = value;
                downloadFilesPath = null;
            }
        }

		/// <summary>
		/// The base folder where files will be downloaded.
		/// </summary>
		public string DownloadFilesBase
		{
			get
            {
                if (downloadFilesPath == null && manifest != null)
                    downloadFilesPath = GetFullDownloadPath();
                return downloadFilesPath != null ? downloadFilesPath : downloadFilesBase;
            }
            set
            {
                downloadFilesBase = value;
                downloadFilesPath = null;
            }
		}

        private string GetFullDownloadPath()
        {
            string path = Path.Combine(RegistryManager.Instance.BasePath, "downloader");
            path = Path.Combine(path, manifest.Application.ApplicationId);
            path = Path.Combine(path, id.ToString());
            return path;
        }

		/// <summary>
		/// An unique identifier for the UpdaterTask.
		/// </summary>
		public Guid TaskId
		{
			get { return id; }
            set { id = value;  }
		}

		/// <summary>
		/// The current state of the UpdaterTask.
		/// <see cref="UpdaterTaskState"/>
		/// </summary>
		public UpdaterTaskState State
		{
			get { return state; }
			set { state = value; }
		} 
		
		#endregion
	}
	
}
