//============================================================================================================
// Microsoft Updater Application Block for .NET
//  http://msdn.microsoft.com/library/en-us/dnbda/html/updater.asp
//	
// Logger.cs
//
// Contains the implementation of the logger helper.
// 
// For more information see the Updater Application Block Implementation Overview. 
// 
//============================================================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//============================================================================================================

using System;

namespace Microsoft.ApplicationBlocks.Mobile.Updater
{
    public enum EventLogEntryType
    {
        Warning,
        Error,
        Information
    }

    /// <summary>
    /// Helper class for logging and exception reporting.
    /// </summary>
    public sealed class Logger
    {
        #region Singleton implementation

        static Logger instance = new Logger();

        private Logger() { }

        /// <summary>
        /// Returns the singleton instance.
        /// </summary>
        private static Logger Instance
        {
            get { return instance; }
        }

        #endregion

        #region Public static inteface

        /// <summary>
        /// Log an error message.
        /// </summary>
        /// <param name="message">The message string.</param>
        public static void LogError(string message)
        {
            Instance.WriteLog(message, EventLogEntryType.Error);
        }

        /// <summary>
        /// Log a warning message.
        /// </summary>
        /// <param name="message">The message string.</param>
        public static void LogWarning(string message)
        {
            Instance.WriteLog(message, EventLogEntryType.Warning);
        }

        /// <summary>
        /// Log an information message.
        /// </summary>
        /// <param name="message">The message string.</param>
        public static void LogInformation(string message)
        {
            Instance.WriteLog(message, EventLogEntryType.Information);
        }

        /// <summary>
        /// Log an exception.
        /// </summary>
        /// <param name="ex">The exception instance.</param>
        public static void LogException(Exception ex)
        {
            LogException(ex, null);
        }

        /// <summary>
        /// Log an exception and a message.
        /// </summary>
        /// <param name="ex">The exception instance.</param>
        /// <param name="message">The message string.</param>
        public static void LogException(Exception ex, string message)
        {
            string log = ex.Message;
            if (!String.IsNullOrEmpty(message))
                log = String.Format("{0}: {1}", message, log);
            LogError(log);
        }

        /// <summary>
        /// Log and throw the exception specified.
        /// </summary>
        /// <param name="ex">The exception instance.</param>
        public static void LogAndThrowException(Exception ex)
        {
            LogAndThrowException(ex, null);
        }

        /// <summary>
        /// Log and throw the exception specified with a message string.
        /// </summary>
        /// <param name="message">The message string.</param>
        /// <param name="ex">The exception instance.</param>
        public static void LogAndThrowException(Exception ex, string message)
        {
            LogException(ex, message);
            throw ex;
        }

        #endregion

        #region Instance members

        /// <summary>
        /// Write the log to EntLib logger.
        /// </summary>
        /// <param name="message">The message string.</param>
        /// <param name="entryType">The event type.</param>
        private void WriteLog(string message, EventLogEntryType entryType)
        {
            // Do nothing
        }

        #endregion
    }
}
