//============================================================================================================
// Microsoft Updater Application Block for .NET
//  http://msdn.microsoft.com/library/en-us/dnbda/html/updater.asp
//	
// FolderDeleteProcessor.cs
//
// Contains the implementation of the FolderDeleteProcessor.
// 
// For more information see the Updater Application Block Implementation Overview. 
// 
//============================================================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//============================================================================================================

using System;
using System.Configuration;
using System.Globalization;
using System.Xml;
using System.IO;
using Microsoft.ApplicationBlocks.Mobile.Updater.Configuration;


namespace Microsoft.ApplicationBlocks.Mobile.Updater.ActivationProcessors
{
	/// <summary>
	/// This processor deletes all the files and subfolders of the specified folder in its configuration.
	/// </summary>
	/// <remarks>
	/// These attributes are defined to configure this processor. They are mandatory except where noted.
	/// <list type="table">
	/// <listheader><term>Attribute</term><description>Description</description></listheader>
	/// <item><term>type</term><description>The type name for this processor</description></item>
	/// <item><term>path</term><description>The path to the folder to delete. Relatives paths are considered to be under the application location folder</description></item>
	/// <item><term>recursive</term><description>Optional. Indicates wheter the subfolders should be recursively deleted. Default is True.</description></item>
	/// </list>
	/// </remarks>
	/// <example>
	/// The following XML excerpt demonstrates how to configure this processor in the manifest file.
	/// <code>
	/// &lt;manifest manifestId="{311085F7-9320-4318-9A67-9BE32F04E933}" mandatory="False" xmlns=... &gt;
	///		...
	///		&lt;activation&gt;
	///			&lt;tasks&gt;
	///				&lt;task type="Microsoft.ApplicationBlocks.Mobile.Updater.ActivationProcessors.FolderDeleteProcessor, Microsoft.ApplicationBlocks.Mobile.Updater.ActivationProcessors" 
	///					path="samples"
	///					recursive="False"
	///				/&gt;
	///				&lt;task type="Microsoft.ApplicationBlocks.Mobile.Updater.ActivationProcessors.FolderDeleteProcessor, Microsoft.ApplicationBlocks.Mobile.Updater.ActivationProcessors" 
	///					path="temp"
	///				/&gt;
	///			&lt;/tasks&gt;
	///		&lt;/activation&gt;
	///	&lt;/manifest&gt;
	/// </code>
	/// </example>
	public class FolderDeleteProcessor : IActivationProcessor
	{
		#region Private members

		/// <summary>
		/// The UpdaterTask provided in the Init method.
		/// </summary>
		private UpdaterTask task;

		/// <summary>
		/// The folder specification to delete.
		/// </summary>
		private string folderSpec;

		/// <summary>
		/// Whether the delete operation is recursive or not.
		/// </summary>
		private bool isRecursive = true;

		#endregion

		#region Constructors

		/// <summary>
		/// Default constructor.
		/// </summary>
		public FolderDeleteProcessor()
		{
		}

		#endregion

		#region IActivationProcessor Members

        /// <summary>
        /// Initializes the processor using the manifest configuration and the UpdaterTask instance.
        /// </summary>
        /// <param name="config">The configuration for the processor in the manifest file.</param>
        /// <param name="task">The UpdaterTask instance.</param>
        /// <remarks>Validates the correctness of the configuration and aborts the activation if it is not valid.</remarks>
        public void Init(Xsd.ActivationProcessorProviderData config, UpdaterTask task)
        {
            this.task = task;

            if (config.AnyAttr == null)
            {
                throw new Exception(Resource.ResourceManager[Resource.MessageKey.ProcessorNotConfigured]);
            }

            foreach (XmlAttribute attr in config.AnyAttr)
            {
                if (String.Compare(attr.Name, "path", false, CultureInfo.InvariantCulture) == 0)
                {
                    folderSpec = attr.Value;
                }
                if (String.Compare(attr.Name, "recursive", false, CultureInfo.InvariantCulture) == 0)
                {
                    isRecursive = bool.Parse(attr.Value);
                }
            }

            if ( String.IsNullOrEmpty(folderSpec) )
            {
                throw new ArgumentException(Resource.ResourceManager[Resource.MessageKey.PathExpected]);
            }
        }

		/// <summary>
		/// Executes the processor.
		/// </summary>
		/// <remarks>Uses the base class library to delete the contents of a folder.</remarks>
		public void Execute()
		{
			Directory.Delete( folderSpec, isRecursive );
		}

		/// <summary>
		/// If the activation fails this method is called to revert the operations performed by the processor.
		/// </summary>
		/// <remarks>No error hadling implemented for this provider.</remarks>
		public void OnError()
		{
		}

		/// <summary>
		/// Prepares the execution and throws an exception if the execution is not possible.
		/// </summary>
        /// <remarks>In keeping with FileCopyProcessor, it completes the source path to a full path.</remarks>
		public void PrepareExecution()
		{
            if (!Path.IsPathRooted(folderSpec))
            {
                folderSpec = Path.Combine(task.Manifest.Application.Location, folderSpec);
            }
        }

		#endregion
	}
}
